@@ -37,7 +37,8 @@ from raven.utils import json, get_versions, get_auth_header, merge_dicts
 from raven._compat import text_type, iteritems
 from raven.utils.encoding import to_unicode
 from raven.utils.serializer import transform
-from raven.utils.stacks import get_stack_info, iter_stack_frames, get_culprit
+from raven.utils.stacks import get_stack_info, iter_stack_frames, slim_string
+from raven.utils.transaction import TransactionStack
 from raven.transport.registry import TransportRegistry, default_transports
 
 # enforce imports to avoid obscure stacktraces with MemoryError
@@ -59,6 +60,13 @@ SDK_VALUE = {
 Raven = None
 
 
+def get_excepthook_client():
+    hook = sys.excepthook
+    client = getattr(hook, 'raven_client', None)
+    if client is not None:
+        return client
+
+
 class ModuleProxyCache(dict):
     def __missing__(self, key):
         module, class_name = key.rsplit('.', 1)
@@ -179,6 +187,12 @@ class Client(object):
         self.tags = o.get('tags') or {}
         self.environment = o.get('environment') or None
         self.release = o.get('release') or os.environ.get('HEROKU_SLUG_COMMIT')
+        self.transaction = TransactionStack()
+        # find the root transaction as the command which launched this
+        # process
+        self.transaction.push(slim_string(' '.join(sys.argv), 128))
+
+        self.ignore_exceptions = set(o.get('ignore_exceptions') or ())
 
         self.module_cache = ModuleProxyCache()
 
@@ -235,8 +249,9 @@ class Client(object):
             __excepthook__ = sys.excepthook
 
         def handle_exception(*exc_info):
-            self.captureException(exc_info=exc_info)
+            self.captureException(exc_info=exc_info, level='fatal')
             __excepthook__(*exc_info)
+        handle_exception.raven_client = self
         sys.excepthook = handle_exception
 
     def install_logging_hook(self):
@@ -400,12 +415,7 @@ class Client(object):
                     )
 
         if not culprit:
-            if 'stacktrace' in data:
-                culprit = get_culprit(data['stacktrace']['frames'])
-            elif 'exception' in data:
-                stacktrace = data['exception']['values'][0].get('stacktrace')
-                if stacktrace:
-                    culprit = get_culprit(stacktrace['frames'])
+            culprit = self.transaction.peek()
 
         if not data.get('level'):
             data['level'] = kwargs.get('level') or logging.ERROR
@@ -589,6 +599,11 @@ class Client(object):
         if exc_info is not None:
             if self.skip_error_for_logging(exc_info):
                 return
+            elif not self.should_capture(exc_info):
+                self.logger.info(
+                    'Not capturing exception due to filters: %s', exc_info[0],
+                    exc_info=sys.exc_info())
+                return
             self.record_exception_seen(exc_info)
 
         data = self.build_msg(
@@ -647,9 +662,9 @@ class Client(object):
             # try to reconstruct a reasonable version of the exception
             for frame in data['exception']['values'][0]['stacktrace']['frames']:
                 output.append('  File "%(fn)s", line %(lineno)s, in %(func)s' % {
-                    'fn': frame['filename'],
-                    'lineno': frame['lineno'],
-                    'func': frame['function'],
+                    'fn': frame.get('filename', 'unknown_filename'),
+                    'lineno': frame.get('lineno', -1),
+                    'func': frame.get('function', 'unknown_function'),
                 })
 
         self.uncaught_logger.error(output)
@@ -764,9 +779,23 @@ class Client(object):
         """
         if exc_info is None or exc_info is True:
             exc_info = sys.exc_info()
+
         return self.capture(
             'raven.events.Exception', exc_info=exc_info, **kwargs)
 
+    def should_capture(self, exc_info):
+        exc_type = exc_info[0]
+        exc_name = '%s.%s' % (exc_type.__module__, exc_type.__name__)
+        exclusions = self.ignore_exceptions
+
+        if exc_type.__name__ in exclusions:
+            return False
+        elif exc_name in exclusions:
+            return False
+        elif any(exc_name.startswith(e[:-1]) for e in exclusions if e.endswith('*')):
+            return False
+        return True
+
     def capture_exceptions(self, function_or_exceptions=None, **kwargs):
         """
         Wrap a function or code block in try/except and automatically call
